package main

import (
	"context"
	"net"
	"net/url"
	"time"

	"gitlab.mashplace.com/mashplace/backend/common/api/health"
	"gitlab.mashplace.com/mashplace/backend/common/generic/addr"
	"gitlab.mashplace.com/mashplace/backend/common/generic/env"
	"gitlab.mashplace.com/mashplace/backend/common/generic/grpc"
	logger "gitlab.mashplace.com/mashplace/backend/common/generic/log"
	"gitlab.mashplace.com/mashplace/backend/common/generic/metrics"
	"gitlab.mashplace.com/mashplace/backend/common/generic/tracer"
	invitation "gitlab.mashplace.com/mashplace/backend/invitation/api"
	"gitlab.mashplace.com/mashplace/backend/invitation/internal/model"
	"gitlab.mashplace.com/mashplace/backend/invitation/internal/model/mongo"
	"gitlab.mashplace.com/mashplace/backend/invitation/internal/model/mongo/migrations/schema"
	"gitlab.mashplace.com/mashplace/backend/invitation/internal/service"
	"gitlab.mashplace.com/mashplace/backend/invitation/internal/task/invitation/expire"
	pendingLandlordDay1 "gitlab.mashplace.com/mashplace/backend/invitation/internal/task/invitation/pending/landlord/day1"
	"gitlab.mashplace.com/mashplace/backend/invitation/version"
	notification "gitlab.mashplace.com/mashplace/backend/notification/api"
	schedule "gitlab.mashplace.com/mashplace/backend/schedule/api"
)

var (
	envSelfRemote          = env.ResolveString("SELF_REMOTE", "mashroom-invitation:10000")
	envNotificationService = notification.NewNotificationServiceClient(
		grpc.Dial(env.ResolveString("NOTIFICATION_SERVICE_URL", "mashroom-notification:10000"), "Notification"),
	)
	envScheduleService = schedule.NewScheduleServiceClient(
		grpc.Dial(env.ResolveString("SCHEDULE_SERVICE_URL", "mashroom-schedule:10000"), "Schedule"),
	)
)

var (
	envMongoURL      = env.ResolveString("MONGO_URL", "mongodb:27017")
	envMongoUsername = env.ResolveString("MONGO_USERNAME", "tenancy-data")
	envMongoPassword = env.ResolveString("MONGO_PASSWORD", "tenancy-data")
	envMongoAuthDB   = env.ResolveString("MONGO_AUTH", "admin")
	envMongoDB       = env.ResolveString("MONGO_DATABASE", "tenancy-data")
	envMongoSSLMode  = env.ResolveBool("MONGO_SSL_MODE", false)
)

var (
	envResendCooldownSMS           = env.ResolveDuration("RESEND_COOLDOWN_SMS", time.Minute)
	envResendCooldownEmail         = env.ResolveDuration("RESEND_COOLDOWN_EMAIL", time.Minute)
	envOTPValidititySMS            = env.ResolveDuration("RESEND_VALIDITIY_SMS", time.Minute*10)
	envOTPValiditityEmail          = env.ResolveDuration("RESEND_VALIDITIY_EMAIL", time.Hour*24)
	envTestOtpPrefix               = env.ResolveString("TEST_OTP_PREFIX", "")
	envReferLandlordGoogleSheetURL = env.ResolveString(
		"REFER_LANDLORD_GOOGLE_SHEET_URL",
		"https://script.google.com/macros/s/"+
			"AKfycbw8AZvAtIwY2sbQd-DrFQMwa4eevLihrGHUnFvpbGNeGzcztJ_4bbijQVxBo03DJF4/exec",
	)
	envInvitationValidityLandlordToManager = env.ResolveDuration(
		"INVITATION_VALIDITY_LANDLORD_TO_MANAGER", time.Hour*24*2,
	)
	envInvitationValidityRelatedToLandlord = env.ResolveDuration(
		"INVITATION_VALIDITY_RELATED_TO_LANDLORD", time.Hour*24*2,
	)
	envInvitationValidityRelatedToTenant = env.ResolveDuration(
		"INVITATION_VALIDITY_RELATED_TO_TENANT", time.Hour*24*7,
	)
)

func scheduledTasks(repository model.Repository) *schedule.Registry {
	taskViewingExpire, err := expire.New(&expire.Config{
		Repository:   repository,
		Notification: envNotificationService,
	})
	if err != nil {
		panic(err)
	}

	taskViewingUpcoming, err := pendingLandlordDay1.New(&pendingLandlordDay1.Config{
		Repository:   repository,
		Notification: envNotificationService,
	})
	if err != nil {
		panic(err)
	}

	return &schedule.Registry{
		Periods: map[string]*schedule.Period{
			"hourly": {
				Interval: time.Hour,
				Handlers: []schedule.Handler{
					taskViewingExpire,
					taskViewingUpcoming,
				},
			},
		},
	}
}

func main() {
	ctx := context.Background()

	defer tracer.Close()

	repository, err := mongo.NewRepository(ctx, &mongo.Config{
		URL:          envMongoURL,
		Username:     envMongoUsername,
		Password:     envMongoPassword,
		Database:     envMongoDB,
		AuthSource:   envMongoAuthDB,
		MongoSSLMode: envMongoSSLMode,
	})
	if err != nil {
		panic(err)
	}

	err = repository.MigrateSchema(ctx, schema.Migrations()...)
	if err != nil {
		panic(err)
	}

	taskRegistry := scheduledTasks(repository)

	cs := service.Connections{
		Notification: envNotificationService,
		Schedule:     envScheduleService,
		SelfRemote:   envSelfRemote,
	}

	referLandlordURL, err := url.Parse(envReferLandlordGoogleSheetURL)
	if err != nil {
		panic(err)
	}

	srv, err := service.NewService(&service.Config{
		Connections:         cs,
		Repository:          repository,
		TaskRegistry:        taskRegistry,
		ResendCooldownSMS:   envResendCooldownSMS,
		ResendCooldownEmail: envResendCooldownEmail,
		InvitationValidity: service.InvitationValidity{
			LandlordToManager: envInvitationValidityLandlordToManager,
			RelatedToTenant:   envInvitationValidityRelatedToTenant,
			RelatedLandlord:   envInvitationValidityRelatedToLandlord,
		},
		OTPValiditySMS:    envOTPValidititySMS,
		OTPValidityEmail:  envOTPValiditityEmail,
		OTPGeneratorSMS:   service.NumberOTPGenerator(5),
		OTPGeneratorEmail: service.NumberOTPGenerator(6),
		TestModePrefix:    envTestOtpPrefix,
		ReferenceSheetURL: *referLandlordURL,
	})
	if err != nil {
		panic(err)
	}

	listener, err := net.Listen("tcp", addr.Grpc)
	if err != nil {
		panic(err)
	}

	grpcServer, err := grpc.NewServer("invitation")
	if err != nil {
		panic(err)
	}

	invitation.RegisterInvitationServiceServer(grpcServer, srv)
	schedule.RegisterReminderConsumerServiceServer(grpcServer, srv)

	logger.Infoln("invitation started")

	healthServer := version.NewHealthServer()
	health.RegisterHealthServiceServer(grpcServer, healthServer)

	go func() {
		_ = metrics.ConfigureListenAndServe(grpcServer)
	}()

	go func() {
		err = srv.SetupSchedule(ctx)
		if err != nil {
			panic(err)
		}
	}()

	err = grpcServer.Serve(listener)
	if err != nil {
		logger.Errorln("gRPC error", err)
	}
}
