package mongo

import (
	"context"
	"time"

	invitation "gitlab.mashplace.com/mashplace/backend/invitation/api"

	"gitlab.mashplace.com/mashplace/backend/common/generic/dbfilter"
	"gitlab.mashplace.com/mashplace/backend/common/generic/dbfilter/mongofilter"
	"gitlab.mashplace.com/mashplace/backend/invitation/internal/model"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
	"go.mongodb.org/mongo-driver/mongo"
	"go.mongodb.org/mongo-driver/mongo/options"
)

type repositoryImplContactValidation struct {
	ContactValidations *mongo.Collection
}

func (repo *repositoryImplContactValidation) ContactValidationSave(
	ctx context.Context,
	cv *model.ContactValidation,
) error {
	if cv.ID.IsZero() {
		cv.ID = primitive.NewObjectID()
		cv.CreatedAt = time.Now()
	}

	cv.UpdatedAt = time.Now()

	_, err := repo.ContactValidations.ReplaceOne(
		ctx,
		bson.M{"_id": cv.ID},
		cv,
		options.Replace().SetUpsert(true),
	)

	return err
}

func (repo *repositoryImplContactValidation) ContactValidationRemoveByID(
	ctx context.Context,
	id string,
) error {
	oid, err := primitive.ObjectIDFromHex(id)
	if err != nil {
		return err
	}

	_, err = repo.ContactValidations.DeleteOne(ctx, bson.M{"_id": oid})

	return err
}

func (repo *repositoryImplContactValidation) ContactValidationGetLatestByParams(
	ctx context.Context,
	typ invitation.ContactValidationType,
	userID, key, remote string,
) (*model.ContactValidation, error) {
	cond := bson.M{
		"type":    typ,
		"user_id": userID,
	}

	if key != "" {
		cond["key"] = key
	} else if remote != "" {
		cond["remote"] = remote
	}

	var cv model.ContactValidation

	err := repo.ContactValidations.FindOne(ctx, cond, options.FindOne().SetSort(bson.M{"expires_at": -1})).Decode(&cv)
	if err != nil {
		return nil, err
	}

	return &cv, nil
}

func (repo *repositoryImplContactValidation) ContactValidationGetByFilters(
	ctx context.Context,
	fs ...dbfilter.Condition,
) (*model.ContactValidation, error) {
	cur, err := mongofilter.Execute(ctx, repo.ContactValidations, dbfilter.And(fs...))
	if err != nil {
		return nil, err
	}

	defer func() {
		_ = cur.Close(ctx)
	}()

	var cv model.ContactValidation

	if cur.Next(ctx) {
		err = cur.Decode(&cv)
		if err != nil {
			return nil, err
		}
	} else {
		return nil, mongo.ErrNoDocuments
	}

	return &cv, nil
}
