package mongo

import (
	"context"
	"time"

	invitation "gitlab.mashplace.com/mashplace/backend/invitation/api"

	"gitlab.mashplace.com/mashplace/backend/common/api/generic/query"
	"gitlab.mashplace.com/mashplace/backend/common/generic/dbfilter"
	"gitlab.mashplace.com/mashplace/backend/common/generic/dbfilter/mongofilter"
	"gitlab.mashplace.com/mashplace/backend/common/generic/mongoquery"
	"gitlab.mashplace.com/mashplace/backend/invitation/internal/model"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
	"go.mongodb.org/mongo-driver/mongo"
	"go.mongodb.org/mongo-driver/mongo/options"
)

type repositoryImplInvitation struct {
	Invitations *mongo.Collection
}

func (repo *repositoryImplInvitation) InvitationSave(
	ctx context.Context,
	invitation *model.Invitation,
) error {
	if invitation.ID.IsZero() {
		invitation.ID = primitive.NewObjectID()
		invitation.CreatedAt = time.Now()
	}

	invitation.UpdatedAt = time.Now()

	_, err := repo.Invitations.ReplaceOne(
		ctx,
		bson.M{"_id": invitation.ID},
		invitation,
		options.Replace().SetUpsert(true),
	)

	return err
}

func (repo *repositoryImplInvitation) InvitationGetByID(
	ctx context.Context,
	id string,
) (*model.Invitation, error) {
	oid, err := primitive.ObjectIDFromHex(id)
	if err != nil {
		return nil, err
	}

	var invitation model.Invitation

	err = repo.Invitations.FindOne(ctx, bson.M{"_id": oid}).Decode(&invitation)
	if err != nil {
		return nil, err
	}

	return &invitation, nil
}

func (repo *repositoryImplInvitation) InvitationListFilter(
	ctx context.Context,
	params *query.QueryParams,
) (ts []*model.Invitation, total int64, err error) {
	filter, opts, err := mongoquery.BuildFilter(model.Invitation{}, params, nil)
	if err != nil {
		return
	}

	cur, err := repo.Invitations.Find(ctx, filter, opts)
	if err != nil {
		return
	}

	err = cur.All(ctx, &ts)
	if err != nil {
		return
	}

	total, err = repo.Invitations.CountDocuments(ctx, filter)
	if err != nil {
		return
	}

	return
}

func (repo *repositoryImplInvitation) InvitationListByFilters(
	ctx context.Context,
	fs ...dbfilter.Condition,
) (vs []*model.Invitation, err error) {
	cur, err := mongofilter.Execute(ctx, repo.Invitations, dbfilter.And(fs...))
	if err != nil {
		return nil, err
	}

	defer func() {
		_ = cur.Close(ctx)
	}()

	err = cur.All(ctx, &vs)
	if err != nil {
		return nil, err
	}

	return
}

func (repo *repositoryImplInvitation) InvitationUpdateNotified(ctx context.Context, id, notification string) error {
	oid, err := primitive.ObjectIDFromHex(id)
	if err != nil {
		return err
	}

	_, err = repo.Invitations.UpdateOne(
		ctx,
		bson.M{"_id": oid},
		bson.M{"$set": bson.M{"notifications." + notification: time.Now()}},
	)

	return err
}

func (repo *repositoryImplInvitation) InvitationUpdateStatus(
	ctx context.Context,
	id string,
	status invitation.InvitationStatus,
) error {
	oid, err := primitive.ObjectIDFromHex(id)
	if err != nil {
		return err
	}

	_, err = repo.Invitations.UpdateOne(
		ctx,
		bson.M{"_id": oid},
		bson.M{"$set": bson.M{"status": status}},
	)

	return err
}

func (repo *repositoryImplInvitation) InvitationGetByFilters(
	ctx context.Context,
	fs ...dbfilter.Condition,
) (*model.Invitation, error) {
	cur, err := mongofilter.Execute(ctx, repo.Invitations, dbfilter.And(fs...))
	if err != nil {
		return nil, err
	}

	defer func() {
		_ = cur.Close(ctx)
	}()

	var cv model.Invitation

	if cur.Next(ctx) {
		err = cur.Decode(&cv)
		if err != nil {
			return nil, err
		}
	} else {
		return nil, mongo.ErrNoDocuments
	}

	return &cv, nil
}

func (repo *repositoryImplInvitation) InvitationUpdateInviteeID(
	ctx context.Context,
	email, userID string,
) (ms []*model.Invitation, err error) {
	filter := bson.M{
		"invitee.email":   email,
		"invitee.user_id": bson.M{"$in": bson.A{"", nil}},
	}

	cur, err := repo.Invitations.Find(
		ctx,
		filter,
	)
	if err != nil {
		return
	}

	err = cur.All(ctx, &ms)
	if err != nil {
		return
	}

	_, err = repo.Invitations.UpdateMany(
		ctx,
		filter,
		bson.M{
			"$set": bson.M{
				"invitee.user_id": userID,
			},
		},
	)
	if err != nil {
		return
	}

	for _, m := range ms {
		m.Invitee.UserID = userID
	}

	return
}
