// Package notifications provides handler for notification status transitions
package notifications

import (
	"context"

	"gitlab.mashplace.com/mashplace/backend/common/api/grpcctx"
	"gitlab.mashplace.com/mashplace/backend/common/generic/grpc"
	invitation "gitlab.mashplace.com/mashplace/backend/invitation/api"
	"gitlab.mashplace.com/mashplace/backend/invitation/internal/model"
	"gitlab.mashplace.com/mashplace/backend/invitation/internal/service/render"
	messaging "gitlab.mashplace.com/mashplace/backend/messaging/api"
	notification "gitlab.mashplace.com/mashplace/backend/notification/api"
)

// HandleInvitationStatusChange sends notifications based on status transition and callback availability
func HandleInvitationStatusChange(
	ctx context.Context,
	notifyclient notification.NotificationServiceClient,
	m *model.Invitation,
) {
	bgctx := grpcctx.ValuesBackgroundContext(ctx)

	var notices []*notification.SendNotificationRequest

	switch m.Status {
	case invitation.InvitationStatus_INVITATION_STATUS_PENDING:
		notices = handleInvitationStatusChangePending(m)
	case invitation.InvitationStatus_INVITATION_STATUS_ACCEPTED:
		notices = handleInvitationStatusChangeAccepted(m)
	case invitation.InvitationStatus_INVITATION_STATUS_DECLINED:
		notices = handleInvitationStatusChangeDeclined(m)
	case invitation.InvitationStatus_INVITATION_STATUS_CANCELED:
		notices = handleInvitationStatusChangeCanceled(m)
	case invitation.InvitationStatus_INVITATION_STATUS_TIMEOUT:
	}

	for _, n := range notices {
		n.Embeds = append(
			n.Embeds,
			&notification.EventEmbed{
				EmbedType:  messaging.EmbedType_EMBED_TYPE_INVITATION,
				EmbedValue: m.ID.Hex(),
			},
			&notification.EventEmbed{
				EmbedType:  messaging.EmbedType_EMBED_TYPE_PROPERTY,
				EmbedValue: m.Property.PropertyID,
			},
			&notification.EventEmbed{
				EmbedType:  messaging.EmbedType_EMBED_TYPE_USER,
				EmbedValue: m.HostID,
			},
		)

		go func(n *notification.SendNotificationRequest) {
			_, _ = notifyclient.SendNotification(bgctx, n)
		}(n)
	}

	HandleInvitationNotify(bgctx, m)
}

// HandleInvitationNotify performs external notifier update with invitation changes
func HandleInvitationNotify(bgctx context.Context, m *model.Invitation) {
	if m.NotifyURL == "" {
		return
	}

	go func() {
		client := invitation.NewInvitationNotifyServiceClient(grpc.Dial(m.NotifyURL, "InvitationNotify"))

		_, _ = client.InvitationChange(grpcctx.WithServiceUser(bgctx), &invitation.InvitationChangeRequest{
			Invitation: render.Invitation(m),
		})
	}()
}

func handleInvitationStatusChangePending(
	m *model.Invitation,
) (notices []*notification.SendNotificationRequest) {
	switch m.Type {
	case invitation.InvitationType_INVITATION_TYPE_RELATED_TO_LANDLORD:
		notices = []*notification.SendNotificationRequest{
			{
				Entity: notification.Entity_ENTITY_PROPERTY_INVITATION,
				Action: notification.Action_ACTION_CREATE,
				Actor: &notification.Actor{
					User: m.HostID,
					Role: notification.Role_ROLE_UNKNOWN,
				},
				Targets: []*notification.Target{
					{
						User: m.Invitee.UserID,
						Role: notification.Role_ROLE_LANDLORD,
						Contacts: &notification.Contacts{
							Email:    m.Invitee.Email,
							Fullname: m.Invitee.Name,
						},
					},
					{
						User: m.HostID,
						Role: notification.Role_ROLE_UNKNOWN,
					},
				},
			},
		}
	case invitation.InvitationType_INVITATION_TYPE_RELATED_TO_TENANT:
		notices = []*notification.SendNotificationRequest{
			{
				Entity: notification.Entity_ENTITY_PROPERTY_INVITATION,
				Action: notification.Action_ACTION_CREATE,
				Actor: &notification.Actor{
					User: m.HostID,
					Role: notification.Role_ROLE_UNKNOWN,
				},
				Targets: []*notification.Target{
					{
						User: m.Invitee.UserID,
						Role: notification.Role_ROLE_TENANT,
						Contacts: &notification.Contacts{
							Email:    m.Invitee.Email,
							Fullname: m.Invitee.Name,
						},
					},
				},
			},
		}
	case invitation.InvitationType_INVITATION_TYPE_LANDLORD_TO_MANAGER:
		notices = []*notification.SendNotificationRequest{
			{
				Entity: notification.Entity_ENTITY_PROPERTY_INVITATION_MANAGER,
				Action: notification.Action_ACTION_CREATE,
				Actor: &notification.Actor{
					User: m.HostID,
					Role: notification.Role_ROLE_LANDLORD,
				},
				Targets: []*notification.Target{
					{
						User: m.Invitee.UserID,
						Role: notification.Role_ROLE_TENANT,
						Contacts: &notification.Contacts{
							Email:    m.Invitee.Email,
							Fullname: m.Invitee.Name,
						},
					},
				},
			},
		}
	}

	return
}

func handleInvitationStatusChangeAccepted(
	m *model.Invitation,
) (notices []*notification.SendNotificationRequest) {
	switch m.Type {
	case invitation.InvitationType_INVITATION_TYPE_RELATED_TO_LANDLORD:
		notices = []*notification.SendNotificationRequest{
			{
				Entity: notification.Entity_ENTITY_PROPERTY_INVITATION_MANAGER,
				Action: notification.Action_ACTION_ACCEPT,
				Actor: &notification.Actor{
					Role: notification.Role_ROLE_LANDLORD,
					User: m.Invitee.UserID,
				},
				Targets: []*notification.Target{
					{
						User: m.Invitee.UserID,
						Role: notification.Role_ROLE_LANDLORD,
					},
					{
						User: m.HostID,
						Role: notification.Role_ROLE_TENANT,
					},
				},
			},
		}
	case invitation.InvitationType_INVITATION_TYPE_LANDLORD_TO_MANAGER:
		notices = []*notification.SendNotificationRequest{
			{
				Entity: notification.Entity_ENTITY_PROPERTY_INVITATION_MANAGER,
				Action: notification.Action_ACTION_ACCEPT,
				Actor: &notification.Actor{
					Role: notification.Role_ROLE_TENANT,
					User: m.Invitee.UserID,
				},
				Targets: []*notification.Target{
					{
						User: m.HostID,
						Role: notification.Role_ROLE_LANDLORD,
					},
					{
						User: m.Invitee.UserID,
						Role: notification.Role_ROLE_TENANT,
					},
				},
			},
		}
	}

	return
}

func handleInvitationStatusChangeDeclined(
	m *model.Invitation,
) (notices []*notification.SendNotificationRequest) {
	switch m.Type {
	case invitation.InvitationType_INVITATION_TYPE_RELATED_TO_LANDLORD:
		notices = []*notification.SendNotificationRequest{
			{
				Entity: notification.Entity_ENTITY_PROPERTY_INVITATION_MANAGER,
				Action: notification.Action_ACTION_REJECT,
				Actor: &notification.Actor{
					Role: notification.Role_ROLE_LANDLORD,
					User: m.Invitee.UserID,
				},
				Targets: []*notification.Target{
					{
						User: m.Invitee.UserID,
						Role: notification.Role_ROLE_LANDLORD,
					},
					{
						User: m.HostID,
						Role: notification.Role_ROLE_TENANT,
					},
				},
			},
		}
	case invitation.InvitationType_INVITATION_TYPE_RELATED_TO_TENANT:
		notices = []*notification.SendNotificationRequest{
			{
				Entity: notification.Entity_ENTITY_PROPERTY_INVITATION_MANAGER,
				Action: notification.Action_ACTION_REJECT,
				Actor: &notification.Actor{
					Role: notification.Role_ROLE_TENANT,
					User: m.Invitee.UserID,
				},
				Targets: []*notification.Target{
					{
						User: m.HostID,
						Role: notification.Role_ROLE_LANDLORD,
					},
				},
			},
		}
	case invitation.InvitationType_INVITATION_TYPE_LANDLORD_TO_MANAGER:
		notices = []*notification.SendNotificationRequest{
			{
				Entity: notification.Entity_ENTITY_PROPERTY_INVITATION_MANAGER,
				Action: notification.Action_ACTION_REJECT,
				Actor: &notification.Actor{
					Role: notification.Role_ROLE_TENANT,
					User: m.Invitee.UserID,
				},
				Targets: []*notification.Target{
					{
						User: m.HostID,
						Role: notification.Role_ROLE_LANDLORD,
					},
					{
						User: m.Invitee.UserID,
						Role: notification.Role_ROLE_TENANT,
					},
				},
			},
		}
	}

	return
}

func handleInvitationStatusChangeCanceled(
	m *model.Invitation,
) (notices []*notification.SendNotificationRequest) {
	if m.Type == invitation.InvitationType_INVITATION_TYPE_LANDLORD_TO_MANAGER {
		notices = []*notification.SendNotificationRequest{
			{
				Entity: notification.Entity_ENTITY_PROPERTY_INVITATION_MANAGER,
				Action: notification.Action_ACTION_CANCEL,
				Actor: &notification.Actor{
					Role: notification.Role_ROLE_LANDLORD,
					User: m.HostID,
				},
				Targets: []*notification.Target{
					{
						User: m.HostID,
						Role: notification.Role_ROLE_LANDLORD,
					},
					{
						User: m.Invitee.UserID,
						Role: notification.Role_ROLE_TENANT,
					},
				},
			},
		}
	}

	return
}
