// Package service provides implementation for API of invitation service
package service

import (
	"context"
	"fmt"
	"math"
	"math/rand"
	"net/url"
	"time"
	"unicode"

	invitation "gitlab.mashplace.com/mashplace/backend/invitation/api"
	"gitlab.mashplace.com/mashplace/backend/invitation/internal/model"
	notification "gitlab.mashplace.com/mashplace/backend/notification/api"
	schedule "gitlab.mashplace.com/mashplace/backend/schedule/api"
)

// Connections for external services
type Connections struct {
	Notification notification.NotificationServiceClient
	Schedule     schedule.ScheduleServiceClient
	SelfRemote   string
}

// OTPGeneratorFunc returns OTP code
type OTPGeneratorFunc func() string

// NumberOTPGenerator created OTPGeneratorFunc returning random number of fixed length
func NumberOTPGenerator(length int) OTPGeneratorFunc {
	lim := int(math.Pow10(length))
	format := fmt.Sprintf("%%0%dd", length)

	return func() string {
		return fmt.Sprintf(format, rand.Intn(lim))
	}
}

// InvitationValidity config
type InvitationValidity struct {
	LandlordToManager time.Duration
	RelatedToTenant   time.Duration
	RelatedLandlord   time.Duration
}

// Resolve invitation duration
func (inv *InvitationValidity) Resolve(t invitation.InvitationType) time.Duration {
	switch t {
	case invitation.InvitationType_INVITATION_TYPE_LANDLORD_TO_MANAGER:
		return inv.LandlordToManager
	case invitation.InvitationType_INVITATION_TYPE_RELATED_TO_LANDLORD:
		return inv.RelatedLandlord
	case invitation.InvitationType_INVITATION_TYPE_RELATED_TO_TENANT:
		return inv.RelatedToTenant
	}

	return 0
}

// Config for property service
type Config struct {
	Connections         Connections
	Repository          model.Repository
	TaskRegistry        *schedule.Registry
	ResendCooldownSMS   time.Duration
	ResendCooldownEmail time.Duration
	InvitationValidity  InvitationValidity
	OTPValiditySMS      time.Duration
	OTPValidityEmail    time.Duration
	OTPGeneratorSMS     OTPGeneratorFunc
	OTPGeneratorEmail   OTPGeneratorFunc
	TestModePrefix      string
	ReferenceSheetURL   url.URL
}

// Service contract for property service
type Service interface {
	invitation.InvitationServiceServer
	schedule.ReminderConsumerServiceServer
	SetupSchedule(ctx context.Context) error
}

// NewService returns new service instance with provided config
func NewService(config *Config) (Service, error) {
	return &serviceImpl{
		Config: *config,
	}, nil
}

type serviceImpl struct {
	Config
}

func (srv *serviceImpl) SetupSchedule(ctx context.Context) error {
	return srv.TaskRegistry.Setup(ctx, srv.Connections.Schedule, "invitation", srv.Connections.SelfRemote)
}

func (srv *serviceImpl) ConsumeReminder(
	ctx context.Context,
	req *schedule.Reminder_ConsumeRequest,
) (*schedule.Reminder_ConsumeResponse, error) {
	err := srv.TaskRegistry.Consume(ctx, req, time.Now())
	if err != nil {
		return nil, err
	}

	return &schedule.Reminder_ConsumeResponse{}, nil
}

func (srv *serviceImpl) generateTestOTP(key string) string {
	if srv.TestModePrefix == "" || key == "" {
		return ""
	}

	result := ""

	for _, v := range key {
		if unicode.IsDigit(v) {
			result = string(v) + result
		}
	}

	return srv.TestModePrefix + result
}
