package service

import (
	"context"
	"time"

	"gitlab.mashplace.com/mashplace/backend/common/api/grpcerr"
	"gitlab.mashplace.com/mashplace/backend/common/generic/log"
	invitation "gitlab.mashplace.com/mashplace/backend/invitation/api"
	"gitlab.mashplace.com/mashplace/backend/invitation/internal/model"
	"gitlab.mashplace.com/mashplace/backend/invitation/internal/service/render"
	messaging "gitlab.mashplace.com/mashplace/backend/messaging/api"
	notification "gitlab.mashplace.com/mashplace/backend/notification/api"
)

func (srv *serviceImpl) ContactValidationCreate(
	ctx context.Context,
	req *invitation.ContactValidationCreateRequest,
) (resp *invitation.ContactValidationCreateResponse, err error) {
	var cooldown, validity time.Duration

	var otp string

	switch req.GetType() {
	case invitation.ContactValidationType_CONTACT_VALIDATION_PHONE:
		cooldown, validity = srv.Config.ResendCooldownSMS, srv.Config.OTPValiditySMS
		otp = srv.Config.OTPGeneratorSMS()
	case invitation.ContactValidationType_CONTACT_VALIDATION_EMAIL:
		cooldown, validity = srv.Config.ResendCooldownEmail, srv.Config.OTPValidityEmail
		otp = srv.Config.OTPGeneratorEmail()
	default:
		return nil, grpcerr.New(grpcerr.InvalidData, "unknown validation type")
	}

	cv, _ := srv.Repository.ContactValidationGetLatestByParams(
		ctx, req.GetType(),
		req.GetUserId(),
		req.GetKey(),
		req.GetRemote(),
	)

	now := time.Now()

	if cv != nil && cv.ExpiresAt.After(now) {
		if !req.GetForce() {
			return &invitation.ContactValidationCreateResponse{
				ContactValidation: render.ContactValidation(cv),
			}, nil
		}

		if time.Since(cv.CreatedAt) < cooldown {
			return nil, grpcerr.New(grpcerr.InvalidData, "resend cooldown is not over yet")
		}
	}

	cv = &model.ContactValidation{
		UserID:    req.GetUserId(),
		Key:       req.GetKey(),
		Otp:       otp,
		Remote:    req.GetRemote(),
		Type:      req.GetType(),
		ExpiresAt: now.Add(validity),
	}

	err = srv.Repository.ContactValidationSave(ctx, cv)
	if err != nil {
		return nil, err
	}

	log.WithFields(log.Fields{
		"id":      cv.ID.Hex(),
		"user_id": cv.UserID,
		"type":    cv.Type,
		"key":     cv.Key,
		"remote":  cv.Remote,
		"otp":     cv.Otp,
	}).Debug("otp generated")

	err = srv.deliverNotification(ctx, cv)
	if err != nil {
		return nil, err
	}

	return &invitation.ContactValidationCreateResponse{
		ContactValidation: render.ContactValidation(cv),
	}, nil
}

func (srv *serviceImpl) deliverNotification(
	ctx context.Context,
	model *model.ContactValidation,
) error {
	contacts := &notification.Contacts{}

	var entity notification.Entity

	switch model.Type {
	case invitation.ContactValidationType_CONTACT_VALIDATION_EMAIL:
		entity = notification.Entity_ENTITY_USER_EMAIL
		contacts.Email = model.Key
	case invitation.ContactValidationType_CONTACT_VALIDATION_PHONE:
		entity = notification.Entity_ENTITY_USER_PHONE
		contacts.Phone = model.Key
	}

	_, err := srv.Connections.Notification.SendNotification(ctx, &notification.SendNotificationRequest{
		Entity: entity,
		Action: notification.Action_ACTION_VERIFY,
		Targets: []*notification.Target{
			{
				Role:     notification.Role_ROLE_AUTHOR,
				Contacts: contacts,
			},
		},
		Embeds: []*notification.EventEmbed{
			{
				EmbedType:  messaging.EmbedType_EMBED_TYPE_TOKEN,
				EmbedValue: model.Otp,
			},
		},
	})

	return err
}

func (srv *serviceImpl) ContactValidationVerify(
	ctx context.Context,
	req *invitation.ContactValidationVerifyRequest,
) (*invitation.ContactValidationVerifyResponse, error) {
	if srv.Config.TestModePrefix != "" && req.GetOtp() != "" && srv.generateTestOTP(req.GetKey()) == req.GetOtp() {
		return &invitation.ContactValidationVerifyResponse{
			Status: invitation.ContactValidationStatus_CONTACT_VALIDATION_STATUS_OK,
		}, nil
	}

	cv, err := srv.Repository.ContactValidationGetLatestByParams(
		ctx, req.GetType(),
		req.GetUserId(),
		req.GetKey(),
		req.GetRemote(),
	)
	if err != nil {
		return nil, err
	}

	now := time.Now()

	if cv.ExpiresAt.Before(now) {
		return &invitation.ContactValidationVerifyResponse{
			Status: invitation.ContactValidationStatus_CONTACT_VALIDATION_STATUS_EXPIRED,
		}, nil
	}

	if cv.Otp != req.GetOtp() {
		return &invitation.ContactValidationVerifyResponse{
			Status: invitation.ContactValidationStatus_CONTACT_VALIDATION_STATUS_FAIL,
		}, nil
	}

	if !req.GetForce() {
		err = srv.Repository.ContactValidationRemoveByID(ctx, cv.ID.Hex())
		if err != nil {
			return nil, err
		}
	}

	return &invitation.ContactValidationVerifyResponse{
		Status: invitation.ContactValidationStatus_CONTACT_VALIDATION_STATUS_OK,
	}, nil
}
