package service

import (
	"context"
	"errors"
	"strings"
	"time"

	"gitlab.mashplace.com/mashplace/backend/common/api/grpcctx"
	"gitlab.mashplace.com/mashplace/backend/common/api/grpcerr"
	"gitlab.mashplace.com/mashplace/backend/common/generic/dbfilter"
	"gitlab.mashplace.com/mashplace/backend/common/generic/log"
	invitation "gitlab.mashplace.com/mashplace/backend/invitation/api"
	"gitlab.mashplace.com/mashplace/backend/invitation/internal/model"
	"gitlab.mashplace.com/mashplace/backend/invitation/internal/notifications"
	"gitlab.mashplace.com/mashplace/backend/invitation/internal/service/render"
	"go.mongodb.org/mongo-driver/bson/primitive"
	"go.mongodb.org/mongo-driver/mongo"
)

var (
	errInvitationCanceled = grpcerr.New(grpcerr.InvalidState, "invitation already canceled")
	errInvitationAccepted = grpcerr.New(grpcerr.InvalidState, "invitation already accepted")
	errInvitationDeclined = grpcerr.New(grpcerr.InvalidState, "invitation already declined")
	errInvitationTimeout  = grpcerr.New(grpcerr.InvalidState, "invitation already timed out")
)

func (srv *serviceImpl) InvitationCreate(
	ctx context.Context,
	req *invitation.InvitationCreateRequest,
) (*invitation.InvitationCreateResponse, error) {
	log.Infof("CreateInvitation request: %+v\n", req)

	expiration := srv.Config.InvitationValidity.Resolve(req.GetType())
	if expiration == 0 {
		return nil, grpcerr.New(grpcerr.InvalidData, "invitation has no period defined: "+req.GetType().String())
	}

	m := &model.Invitation{
		ID:        primitive.NewObjectID(),
		CreatedAt: time.Now(),
		Type:      req.GetType(),
		HostID:    req.GetHostId(),
		Invitee: model.User{
			UserID: req.GetInvitee().GetUserId(),
			Email:  strings.ToLower(req.GetInvitee().GetEmail()),
			Name:   req.GetInvitee().GetName(),
			Phone:  req.GetInvitee().GetPhone(),
		},
		Otp: srv.OTPGeneratorEmail(),
		Property: model.Property{
			PropertyID: req.GetProperty().GetPropertyId(),
			VersionID:  req.GetProperty().GetVersionId(),
		},
		Status:    invitation.InvitationStatus_INVITATION_STATUS_PENDING,
		ExpiresAt: time.Now().Add(expiration),
		Offer:     req.GetOffer(),
		NotifyURL: req.GetNotifyCallback(),
		Message:   req.GetMessage(),
	}

	if req.GetType() == invitation.InvitationType_INVITATION_TYPE_LANDLORD_TO_MANAGER {
		err := srv.handleLandlordToManagerInvitationCreated(ctx, req.GetProperty().GetPropertyId(), req.GetHostId())
		if err != nil {
			log.WithError(err).Errorf("handleLandlordToManagerInvitationCreated failed: %+v", m)

			return nil, err
		}
	}

	err := srv.Repository.InvitationSave(ctx, m)
	if err != nil {
		log.WithError(err).Errorf("CreateInvitation model creation failed: %+v", m)

		return nil, err
	}

	log.Debugf("CreateInvitation model created: %+v", m)

	notifications.HandleInvitationStatusChange(ctx, srv.Connections.Notification, m)

	return &invitation.InvitationCreateResponse{
		Invitation: render.Invitation(m),
	}, nil
}

func (srv *serviceImpl) handleLandlordToManagerInvitationCreated(
	ctx context.Context,
	property, host string,
) error {
	invs, err := srv.Repository.InvitationListByFilters(
		ctx,
		dbfilter.Eq("property.id", property),
		dbfilter.Eq("type", invitation.InvitationType_INVITATION_TYPE_LANDLORD_TO_MANAGER),
		dbfilter.In("status",
			invitation.InvitationStatus_INVITATION_STATUS_PENDING,
			invitation.InvitationStatus_INVITATION_STATUS_ACCEPTED,
		),
		dbfilter.Eq("host_id", host),
	)
	if err != nil && !errors.Is(err, mongo.ErrNoDocuments) {
		return err
	}

	for _, inv := range invs {
		inv.Status = invitation.InvitationStatus_INVITATION_STATUS_CANCELED

		err = srv.Repository.InvitationUpdateStatus(ctx, inv.ID.Hex(), inv.Status)
		if err != nil {
			return err
		}

		notifications.HandleInvitationStatusChange(ctx, srv.Connections.Notification, inv)
	}

	return nil
}

func (srv *serviceImpl) InvitationGet(
	ctx context.Context,
	req *invitation.InvitationGetRequest,
) (*invitation.InvitationGetResponse, error) {
	inv, err := srv.Repository.InvitationGetByID(ctx, req.GetId())
	if err != nil {
		return nil, err
	}

	return &invitation.InvitationGetResponse{
		Invitation: render.Invitation(inv),
	}, nil
}

func (srv *serviceImpl) InvitationAccept(
	ctx context.Context,
	req *invitation.InvitationAcceptRequest,
) (*invitation.InvitationAcceptResponse, error) {
	inv, err := srv.Repository.InvitationGetByID(ctx, req.GetId())
	if err != nil {
		return nil, err
	}

	switch inv.Status {
	case invitation.InvitationStatus_INVITATION_STATUS_ACCEPTED:
		return &invitation.InvitationAcceptResponse{
			Invitation: render.Invitation(inv),
		}, nil
	case invitation.InvitationStatus_INVITATION_STATUS_CANCELED:
		return nil, errInvitationCanceled
	case invitation.InvitationStatus_INVITATION_STATUS_TIMEOUT:
		return nil, errInvitationTimeout
	}

	inv.Status = invitation.InvitationStatus_INVITATION_STATUS_ACCEPTED

	if inv.Invitee.UserID == "" {
		inv.Invitee.UserID = req.GetUserId()
	}

	err = srv.Repository.InvitationSave(ctx, inv)
	if err != nil {
		return nil, err
	}

	notifications.HandleInvitationStatusChange(ctx, srv.Connections.Notification, inv)

	return &invitation.InvitationAcceptResponse{
		Invitation: render.Invitation(inv),
	}, nil
}

func (srv *serviceImpl) InvitationDecline(
	ctx context.Context,
	req *invitation.InvitationDeclineRequest,
) (*invitation.InvitationDeclineResponse, error) {
	inv, err := srv.Repository.InvitationGetByID(ctx, req.GetId())
	if err != nil {
		return nil, err
	}

	switch inv.Status {
	case invitation.InvitationStatus_INVITATION_STATUS_DECLINED:
		return &invitation.InvitationDeclineResponse{
			Invitation: render.Invitation(inv),
		}, nil
	case invitation.InvitationStatus_INVITATION_STATUS_CANCELED:
		return nil, errInvitationCanceled
	case invitation.InvitationStatus_INVITATION_STATUS_TIMEOUT:
		return nil, errInvitationTimeout
	}

	inv.Status = invitation.InvitationStatus_INVITATION_STATUS_DECLINED

	if inv.Invitee.UserID == "" {
		inv.Invitee.UserID = req.GetUserId()
	}

	err = srv.Repository.InvitationSave(ctx, inv)
	if err != nil {
		return nil, err
	}

	notifications.HandleInvitationStatusChange(ctx, srv.Connections.Notification, inv)

	return &invitation.InvitationDeclineResponse{
		Invitation: render.Invitation(inv),
	}, nil
}

func (srv *serviceImpl) InvitationCancel(
	ctx context.Context,
	req *invitation.InvitationCancelRequest,
) (*invitation.InvitationCancelResponse, error) {
	inv, err := srv.Repository.InvitationGetByFilters(
		ctx,
		dbfilter.Eq("_id", dbfilter.ID(req.GetId())),
		dbfilter.Eq("host_id", req.GetUserId()),
	)
	if err != nil {
		return nil, err
	}

	switch inv.Status {
	case invitation.InvitationStatus_INVITATION_STATUS_CANCELED:
		return &invitation.InvitationCancelResponse{
			Invitation: render.Invitation(inv),
		}, nil
	case invitation.InvitationStatus_INVITATION_STATUS_ACCEPTED:
		if inv.Type != invitation.InvitationType_INVITATION_TYPE_LANDLORD_TO_MANAGER {
			return nil, errInvitationAccepted
		}
	case invitation.InvitationStatus_INVITATION_STATUS_DECLINED:
		return nil, errInvitationDeclined
	case invitation.InvitationStatus_INVITATION_STATUS_TIMEOUT:
		return nil, errInvitationTimeout
	}

	inv.Status = invitation.InvitationStatus_INVITATION_STATUS_CANCELED

	err = srv.Repository.InvitationUpdateStatus(ctx, inv.ID.Hex(), inv.Status)
	if err != nil {
		return nil, err
	}

	notifications.HandleInvitationStatusChange(ctx, srv.Connections.Notification, inv)

	return &invitation.InvitationCancelResponse{
		Invitation: render.Invitation(inv),
	}, nil
}

func (srv *serviceImpl) InvitationCancelAll(
	ctx context.Context,
	req *invitation.InvitationCancelAllRequest,
) (*invitation.InvitationCancelAllResponse, error) {
	invs, err := srv.Repository.InvitationListByFilters(
		ctx,
		dbfilter.Eq("property.id", req.GetPropertyId()),
		dbfilter.Eq("status", invitation.InvitationStatus_INVITATION_STATUS_PENDING),
		dbfilter.Eq("host_id", req.GetUserId()),
	)
	if err != nil {
		return nil, err
	}

	for _, inv := range invs {
		inv.Status = invitation.InvitationStatus_INVITATION_STATUS_CANCELED

		err = srv.Repository.InvitationUpdateStatus(ctx, inv.ID.Hex(), inv.Status)
		if err != nil {
			return nil, err
		}

		notifications.HandleInvitationStatusChange(ctx, srv.Connections.Notification, inv)
	}

	return &invitation.InvitationCancelAllResponse{}, nil
}

func (srv *serviceImpl) InvitationListIncoming(
	ctx context.Context,
	req *invitation.InvitationListIncomingRequest,
) (*invitation.InvitationListIncomingResponse, error) {
	conds := []dbfilter.Condition{
		dbfilter.Eq("invitee.email", req.GetEmail()),
	}

	if req.GetPropertyId() != "" {
		conds = append(conds, dbfilter.Eq("property.property_id", req.GetPropertyId()))
	}

	invs, err := srv.Repository.InvitationListByFilters(ctx, conds...)
	if err != nil {
		return nil, err
	}

	return &invitation.InvitationListIncomingResponse{
		Invitations: render.Invitations(invs),
	}, nil
}

func (srv *serviceImpl) InvitationListOutgoing(
	ctx context.Context,
	req *invitation.InvitationListOutgoingRequest,
) (*invitation.InvitationListOutgoingResponse, error) {
	invs, err := srv.Repository.InvitationListByFilters(ctx, dbfilter.Eq("host_id", req.GetUserId()))
	if err != nil {
		return nil, err
	}

	return &invitation.InvitationListOutgoingResponse{
		Invitations: render.Invitations(invs),
	}, nil
}

func (srv *serviceImpl) InvitationFilterList(
	ctx context.Context,
	req *invitation.InvitationFilterListRequest,
) (*invitation.InvitationFilterListResponse, error) {
	invs, total, err := srv.Repository.InvitationListFilter(ctx, req.GetParams())
	if err != nil {
		return nil, err
	}

	return &invitation.InvitationFilterListResponse{
		Invitations: render.Invitations(invs),
		Total:       int32(total),
	}, nil
}

func (srv *serviceImpl) InvitationUpdateInviteeID(
	ctx context.Context,
	req *invitation.InvitationUpdateInviteeIDRequest,
) (*invitation.InvitationUpdateInviteeIDResponse, error) {
	ms, err := srv.Repository.InvitationUpdateInviteeID(ctx, req.GetEmail(), req.GetInviteeId())
	if err != nil {
		return nil, err
	}

	bgctx := grpcctx.ValuesBackgroundContext(ctx)

	for _, inv := range ms {
		notifications.HandleInvitationNotify(bgctx, inv)
	}

	return &invitation.InvitationUpdateInviteeIDResponse{}, nil
}
