// Package day1 provides task for notifying registered landlords they have incoming property invitation
package day1

import (
	"context"
	"fmt"
	"time"

	"gitlab.mashplace.com/mashplace/backend/common/api/grpcctx"
	"gitlab.mashplace.com/mashplace/backend/common/generic/dbfilter"
	"gitlab.mashplace.com/mashplace/backend/common/generic/log"
	invitation "gitlab.mashplace.com/mashplace/backend/invitation/api"
	"gitlab.mashplace.com/mashplace/backend/invitation/internal/model"
	messaging "gitlab.mashplace.com/mashplace/backend/messaging/api"
	notification "gitlab.mashplace.com/mashplace/backend/notification/api"
	schedule "gitlab.mashplace.com/mashplace/backend/schedule/api"
	"google.golang.org/protobuf/types/known/durationpb"
)

// Config for a task
type Config struct {
	Repository   model.Repository
	Notification notification.NotificationServiceClient
}

// New returns new task instance
func New(config *Config) (schedule.Handler, error) {
	return &handlerImpl{
		Config: *config,
	}, nil
}

type handlerImpl struct {
	Config
}

func (h *handlerImpl) Name() string {
	return "invitation-pending-landlord-day1"
}

func (h *handlerImpl) Consume(
	ctx context.Context,
	req *schedule.Reminder_ConsumeRequest,
	data []byte,
	now time.Time,
) error {
	bgctx := grpcctx.ValuesBackgroundContext(ctx)

	go h.process(bgctx, now)

	return nil
}

func (h *handlerImpl) process(ctx context.Context, now time.Time) {
	models, err := h.Repository.InvitationListByFilters(
		ctx,
		dbfilter.Eq("type", invitation.InvitationType_INVITATION_TYPE_RELATED_TO_LANDLORD),
		dbfilter.Eq("status", invitation.InvitationStatus_INVITATION_STATUS_PENDING),
		dbfilter.Not(dbfilter.Eq("invitee.user_id", "")),
		dbfilter.Lte("created_at", now.Add(-24*time.Hour)),
		dbfilter.Gt("created_at", now.Add(-(24+12)*time.Hour)),
		dbfilter.Not(dbfilter.Gt("notifications.landlord_pending_over_24h", time.Time{})),
	)
	if err != nil {
		return
	}

	for _, m := range models {
		_, err = h.Notification.SendNotification(ctx, &notification.SendNotificationRequest{
			Actor: &notification.Actor{
				User: m.HostID,
				Role: notification.Role_ROLE_TENANT,
			},
			Entity: notification.Entity_ENTITY_PROPERTY_INVITATION_MANAGER,
			Action: notification.Action_ACTION_REJECT,
			Targets: []*notification.Target{
				{
					User: m.Invitee.UserID,
					Role: notification.Role_ROLE_LANDLORD,
				},
			},
			Duration: durationpb.New(time.Hour),
			Embeds: []*notification.EventEmbed{
				{
					EmbedType:  messaging.EmbedType_EMBED_TYPE_INVITATION,
					EmbedValue: m.ID.Hex(),
				},
				{
					EmbedType:  messaging.EmbedType_EMBED_TYPE_PROPERTY,
					EmbedValue: m.Property.PropertyID,
				},
				{
					EmbedType:  messaging.EmbedType_EMBED_TYPE_USER,
					EmbedValue: m.HostID,
				},
			},
		})
		if err != nil {
			log.WithError(fmt.Errorf("failed to send: %w", err)).Error("sending notification")

			continue
		}

		_ = h.Repository.InvitationUpdateNotified(ctx, m.ID.Hex(), "landlord_pending_over_24h")
	}
}
